/*
 *  Build and use process:
 *    cd /usr/src/redhat/BUILD/kernel-2.6.18/linux-2.6.18-398.el5.freeflow.x86_64/
 *    make M=drivers/usb/input
 *    echo -n 2-1.4.1.3:1.0 > /sys/bus/usb/drivers/freeflow/unbind
 *    echo -n 2-1.4.1.3:1.0 > /sys/bus/usb/drivers/usbhid/bind
 *    /sbin/rmmod freeflow
 *    cp drivers/usb/input/freeflow.ko /lib/modules/2.6.18-398.el5.ff2/kernel/drivers/usb/input
 *    /sbin/modprobe freeflow
 *    echo -n 2-1.4.1.3:1.0 > /sys/bus/usb/drivers/usbhid/unbind
 *    echo -n 2-1.4.1.3:1.0 > /sys/bus/usb/drivers/freeflow/bind
 *    dmesg | grep freeflow | more
 *
 *  Native support for the Aiptek HyperPen USB Tablets
 *  (4000U/5000U/6000U/8000U/12000U)
 *
 *  Copyright (c) 2001      Chris Atenasio   <chris@crud.net>
 *  Copyright (c) 2002-2004 Bryan W. Headley <bwheadley@earthlink.net>
 *
 *  based on wacom.c by
 *     Vojtech Pavlik      <vojtech@suse.cz>
 *     Andreas Bach Aaen   <abach@stofanet.dk>
 *     Clifford Wolf       <clifford@clifford.at>
 *     Sam Mosel           <sam.mosel@computer.org>
 *     James E. Blair      <corvus@gnu.org>
 *     Daniel Egger        <egger@suse.de>
 *
 *  Many thanks to Oliver Kuechemann for his support.
 *
 *  ChangeLog:
 *
 * NOTE:
 *      This kernel driver is augmented by the "Aiptek" XFree86 input
 *      driver for your X server, as well as the Gaiptek GUI Front-end
 *      "Tablet Manager".
 *      These three products are highly interactive with one another,
 *      so therefore it's easier to document them all as one subsystem.
 *      Please visit the project's "home page", located at,
 *      http://aiptektablet.sourceforge.net.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <linux/jiffies.h>
#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/module.h>
#include <linux/init.h>
#include <linux/usb/input.h>
#include <linux/sched.h>
#include <asm/uaccess.h>
#include <asm/unaligned.h>

/*
 * Version Information
 */
#define DRIVER_VERSION "F1.2 (2.12.14/09:10)"
#define DRIVER_AUTHOR  "Pete Arnold/Adder Technology Ltd"
#define DRIVER_DESC    "Free-Flow USB Mouse Driver (Linux 2.6.x)"

#define USB_VENDOR_ID_ADDER               0x21d1

    /* Length of incoming packet from the tablet
     */
#define AIPTEK_PACKET_LENGTH              8

struct aiptek_features {
    char usbPath[64 + 1];       /* device's physical usb path     */
    char inputPath[64 + 1];     /* input device path              */
};

struct aiptek {
    struct input_dev*           inputdev;               /* input device struct           */
    struct usb_device*          usbdev;                 /* usb device struct             */
    struct urb*                 urb;                    /* urb for incoming reports      */
    dma_addr_t                  data_dma;               /* our dma stuffage              */
    struct aiptek_features      features;               /* tablet's array of features    */
    unsigned int                ifnum;                  /* interface number for IO       */
    unsigned long               eventCount;             /* event count                   */
    unsigned char*              data;                   /* incoming packet data          */
};

/***********************************************************************
 * aiptek_irq can receive one of six potential reports.
 * The documentation for each is in the body of the function.
 *
 * The tablet reports on several attributes per invocation of
 * aiptek_irq. Because the Linux Input Event system allows the
 * transmission of ONE attribute per input_report_xxx() call,
 * collation has to be done on the other end to reconstitute
 * a complete tablet report. Further, the number of Input Event reports
 * submitted varies, depending on what USB report type, and circumstance.
 * To deal with this, EV_MSC is used to indicate an 'end-of-report'
 * message. This has been an undocumented convention understood by the kernel
 * tablet driver and clients such as gpm and XFree86's tablet drivers.
 *
 * Of the information received from the tablet, the one piece I
 * cannot transmit is the proximity bit (without resorting to an EV_MSC
 * convention above.) I therefore have taken over REL_MISC and ABS_MISC
 * (for relative and absolute reports, respectively) for communicating
 * Proximity. Why two events? I thought it interesting to know if the
 * Proximity event occurred while the tablet was in absolute or relative
 * mode.
 *
 * Other tablets use the notion of a certain minimum stylus pressure
 * to infer proximity. While that could have been done, that is yet
 * another 'by convention' behavior, the documentation for which
 * would be spread between two (or more) pieces of software.
 *
 * EV_MSC usage was terminated for this purpose in Linux 2.5.x, and
 * replaced with the input_sync() method (which emits EV_SYN.)
 */

static void aiptek_irq(struct urb *urb, struct pt_regs *regs)
{
    struct aiptek *aiptek = urb->context;
    unsigned char *data = aiptek->data;
    struct input_dev *inputdev = aiptek->inputdev;
    int retval, x, y, screen;

    printk(KERN_INFO "freeflow - irq.\n");
    printk(KERN_INFO "urb transfer buffer length=%d.\n", urb->transfer_buffer_length);

    switch (urb->status) {
    case 0:
        /* Success */
        break;
    case -EOVERFLOW:
        /* Temporary hack for overflow error. */
        break;
    case -ECONNRESET:
    case -ENOENT:
    case -ESHUTDOWN:
        /* This urb is terminated, clean up */
        dbg("%s - urb shutting down with status: %d",
            __FUNCTION__, urb->status);
        return;

    default:
        dbg("%s - nonzero urb status received: %d",
            __FUNCTION__, urb->status);
        goto exit;
    }

    aiptek->eventCount++;

    /* Report 99's come from the mouse in absolute mode.
     */
    if (data[0] == 99) {
        input_regs(inputdev, regs);
        x = le16_to_cpu(get_unaligned((__le16 *) (data + 1)));
        y = le16_to_cpu(get_unaligned((__le16 *) (data + 3)));
        screen = data[5];

        input_report_abs(inputdev, ABS_X, x);
        input_report_abs(inputdev, ABS_Y, y);
        input_report_abs(inputdev, ABS_MISC, screen);
        input_sync(inputdev);

    } else {
        dbg("Unknown report %d", data[0]);
    }

exit:
    retval = usb_submit_urb(urb, GFP_ATOMIC);
    if (retval != 0) {
        err("%s - usb_submit_urb failed with result %d",
            __FUNCTION__, retval);
    }
}

/***********************************************************************
 * These are the USB id's known so far. We do not identify them to
 * specific Aiptek model numbers, because there has been overlaps,
 * use, and reuse of id's in existing models. Certain models have
 * been known to use more than one ID, indicative perhaps of
 * manufacturing revisions. In any event, we consider these
 * IDs to not be model-specific nor unique.
 */
static const struct usb_device_id freeflow_ids[] = {
    {USB_DEVICE(USB_VENDOR_ID_ADDER, 0x01)},
    {}
};

MODULE_DEVICE_TABLE(usb, freeflow_ids);

/***********************************************************************
 * Open an instance of the freeflow driver.
 */
static int freeflow_open(struct input_dev *inputdev)
{
    struct aiptek *aiptek = inputdev->private;

    printk(KERN_INFO "freeflow - open.\n");

    aiptek->urb->dev = aiptek->usbdev;
    if (usb_submit_urb(aiptek->urb, GFP_KERNEL) != 0)
        return -EIO;

    return 0;
}

/***********************************************************************
 * Close an instance of the freeflow driver.
 */
static void freeflow_close(struct input_dev *inputdev)
{
    struct aiptek *aiptek = inputdev->private;

    printk(KERN_INFO "freeflow - close.\n");

    usb_kill_urb(aiptek->urb);
}

/***********************************************************************
 * This routine is called when a tablet has been identified. It basically
 * sets up the tablet and the driver's internal structures.
 */
static int
aiptek_probe(struct usb_interface *intf, const struct usb_device_id *id)
{
    struct usb_device *usbdev = interface_to_usbdev(intf);
    struct usb_endpoint_descriptor *endpoint;
    struct aiptek *aiptek;
    struct input_dev *inputdev;
    struct input_handle *inputhandle;
    struct list_head *node, *next;

    printk(KERN_INFO "freeflow - probe.\n");

    aiptek = kzalloc(sizeof(struct aiptek), GFP_KERNEL);
    inputdev = input_allocate_device();
    if (!aiptek || !inputdev)
        goto fail1;

    aiptek->data = usb_buffer_alloc(usbdev, AIPTEK_PACKET_LENGTH,
                    SLAB_ATOMIC, &aiptek->data_dma);
    if (!aiptek->data)
        goto fail1;

    aiptek->urb = usb_alloc_urb(0, GFP_KERNEL);
    if (!aiptek->urb)
        goto fail2;

    aiptek->inputdev = inputdev;
    aiptek->usbdev = usbdev;
    aiptek->ifnum = intf->altsetting[0].desc.bInterfaceNumber;

    /* Determine the usb devices' physical path.
     * Asketh not why we always pretend we're using "../input0",
     * but I suspect this will have to be refactored one
     * day if a single USB device can be a keyboard & a mouse
     * & a tablet, and the inputX number actually will tell
     * us something...
     */
    usb_make_path(usbdev, aiptek->features.usbPath,
            sizeof(aiptek->features.usbPath));
    strlcat(aiptek->features.usbPath, "/input0",
        sizeof(aiptek->features.usbPath));

    /* Set up client data, pointers to open and close routines
     * for the input device.
     */
    inputdev->name = "Aiptek";
    inputdev->phys = aiptek->features.usbPath;
    usb_to_input_id(usbdev, &inputdev->id);
    inputdev->cdev.dev = &intf->dev;
    inputdev->private = aiptek;
    inputdev->open = freeflow_open;
    inputdev->close = freeflow_close;

    /* Now program the capacities of the tablet, in terms of being
     * an input device.
     */
    inputdev->evbit[0] |= BIT(EV_KEY)
        | BIT(EV_ABS)
        | BIT(EV_REL)
        | BIT(EV_MSC);

    inputdev->absbit[0] |= BIT(ABS_MISC);

    inputdev->relbit[0] |=
        (BIT(REL_X) | BIT(REL_Y) | BIT(REL_WHEEL) | BIT(REL_MISC));

    inputdev->keybit[LONG(BTN_LEFT)] |=
        (BIT(BTN_LEFT) | BIT(BTN_RIGHT) | BIT(BTN_MIDDLE));

    inputdev->keybit[LONG(BTN_DIGI)] |=
        (BIT(BTN_TOOL_PEN) |
         BIT(BTN_TOOL_RUBBER) |
         BIT(BTN_TOOL_PENCIL) |
         BIT(BTN_TOOL_AIRBRUSH) |
         BIT(BTN_TOOL_BRUSH) |
         BIT(BTN_TOOL_MOUSE) |
         BIT(BTN_TOOL_LENS) |
         BIT(BTN_TOUCH) | BIT(BTN_STYLUS) | BIT(BTN_STYLUS2));

    inputdev->mscbit[0] = BIT(MSC_SERIAL);

    /*
     * Program the input device coordinate capacities. We do not yet
     * know what maximum X, Y, and Z values are, so we're putting fake
     * values in. Later, we'll ask the tablet to put in the correct
     * values.
     */
    input_set_abs_params(inputdev, ABS_X, 0, 32767, 0, 0);
    input_set_abs_params(inputdev, ABS_Y, 0, 32767, 0, 0);
    input_set_abs_params(inputdev, ABS_MISC, 0, 1, 0, 0);
    endpoint = &intf->altsetting[0].endpoint[0].desc;

    /* Go set up our URB, which is called when the tablet receives
     * input.
     */
    usb_fill_int_urb(aiptek->urb,
             aiptek->usbdev,
             usb_rcvintpipe(aiptek->usbdev,
                    endpoint->bEndpointAddress),
             aiptek->data, 8, aiptek_irq, aiptek,
             endpoint->bInterval);

    aiptek->urb->transfer_dma = aiptek->data_dma;
    aiptek->urb->transfer_flags |= URB_NO_TRANSFER_DMA_MAP;

    /* Register the tablet as an Input Device
     */
    input_register_device(aiptek->inputdev);

    /* We now will look for the evdev device which is mapped to
     * the tablet. The partial name is kept in the link list of
     * input_handles associated with this input device.
     * What identifies an evdev input_handler is that it begins
     * with 'event', continues with a digit, and that in turn
     * is mapped to input/eventN.
     */
    list_for_each_safe(node, next, &inputdev->h_list) {
        inputhandle = to_handle(node);
        if (strncmp(inputhandle->name, "event", 5) == 0) {
            strcpy(aiptek->features.inputPath, inputhandle->name);
            break;
        }
    }

    /* Associate this driver's struct with the usb interface.
     */
    usb_set_intfdata(intf, aiptek);

#if 0
    /* Set up the sysfs files
     */
    aiptek_add_files(&intf->dev);
#endif

    /* Make sure the evdev module is loaded. Assuming evdev IS a module :-)
     */
    if (request_module("evdev") != 0)
        info("aiptek: error loading 'evdev' module");

    return 0;

fail2:    usb_buffer_free(usbdev, AIPTEK_PACKET_LENGTH, aiptek->data,
            aiptek->data_dma);
fail1:    input_free_device(inputdev);
    kfree(aiptek);
    return -ENOMEM;
}

/* Forward declaration */
static void aiptek_disconnect(struct usb_interface *intf);

static struct usb_driver freeflow_driver = {
    .name = "freeflow",
    .probe = aiptek_probe,
    .disconnect = aiptek_disconnect,
    .id_table = freeflow_ids,
};

/***********************************************************************
 * Deal with tablet disconnecting from the system.
 */
static void aiptek_disconnect(struct usb_interface *intf)
{
    struct aiptek *aiptek = usb_get_intfdata(intf);

    printk(KERN_INFO "freeflow - disconnect.\n");

    /* Disassociate driver's struct with usb interface
     */
    usb_set_intfdata(intf, NULL);
    if (aiptek != NULL) {
        /* Free & unhook everything from the system.
         */
        usb_kill_urb(aiptek->urb);
        input_unregister_device(aiptek->inputdev);
#if 0
        aiptek_delete_files(&intf->dev);
#endif
        usb_free_urb(aiptek->urb);
        usb_buffer_free(interface_to_usbdev(intf),
                AIPTEK_PACKET_LENGTH,
                aiptek->data, aiptek->data_dma);
        kfree(aiptek);
    }
}

static int __init freeflow_init(void)
{
    int result = usb_register(&freeflow_driver);

    printk(KERN_INFO "freeflow kernel driver "DRIVER_VERSION" - init.\n");

    if (result == 0) {
        info(DRIVER_VERSION ": " DRIVER_AUTHOR);
        info(DRIVER_DESC);
    }
    return result;
}

static void __exit freeflow_exit(void)
{
    printk(KERN_INFO "freeflow - exit.\n");

    usb_deregister(&freeflow_driver);
}

MODULE_AUTHOR(DRIVER_AUTHOR);
MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_LICENSE("GPL");

#if 0
module_param(programmableDelay, int, 0);
MODULE_PARM_DESC(programmableDelay, "delay used during tablet programming");
module_param(jitterDelay, int, 0);
MODULE_PARM_DESC(jitterDelay, "stylus/mouse settlement delay");
#endif
module_init(freeflow_init);
module_exit(freeflow_exit);